#ifndef __MENU__
#define __MENU__

#include "ll.h"
#include "options.h"

#define MENU_OP_RENDER     0
#define MENU_OP_GETSIZE    1
#define MENU_OP_CLICKED    2
#define MENU_OP_SELECTED   3
#define MENU_OP_UNSELECTED 4
#define MENU_OP_DESTROY    5

#define MENU_DF_NORMAL 0
#define MENU_DF_SELECTED  (1 << 1)
#define MENU_DF_SUBMENU   (1 << 2)
#define MENU_DF_SEPARATOR (1 << 3)

#define MENU_SHOW_TOP_LEFT 0
#define MENU_SHOW_BOTTOM_LEFT 1

typedef enum {MENUITEM_NORMAL,MENUITEM_SUBMENU,MENUITEM_SEPARATOR} MENUITEMTYPE;
typedef struct _tagMENUOPERATION
{
  int cmd;
  union
  {
    struct 
    {
      unsigned int width;
      unsigned int height;
      unsigned int drawFlags;
    } size;
    struct
    {
      Drawable drawable;
      unsigned int drawFlags;
      int x;
      int y;
      int width;
      int height;
    } draw;
  } data;
} MENUOPERATION,*LPMENUOPERATION;

typedef void (*MENUCALLBACK)(LPMENUOPERATION lpmo,void *param);

typedef struct _tagMENUITEM
{
  NODE node;
  MENUCALLBACK callback;
  MENUITEMTYPE mit;
  void *param;
  struct _tagMENU *pSubMenu;
  int x;
  int y;
  unsigned int width;
  unsigned int height;
} MENUITEM,*LPMENUITEM;

typedef struct _tagMENU
{
  Window wnd;
  int shown;
  int focus;
  int background_pixel;
  unsigned int width;
  unsigned int height;
  LPLIST pList;
  LPMENUITEM pSelected;
  struct _tagMENU *pParent;
} MENU,*LPMENU;

typedef struct _tagMENUOPTIONS
{
  Window parent;
  unsigned int background;
} MENUOPTIONS,*LPMENUOPTIONS;

LPMENU CreateMenu(LPMENUOPTIONS lpmop);
LPMENUITEM AddMenuItem(LPMENU pMenu,MENUCALLBACK callback,void *param,MENUITEMTYPE mit);
LPMENU AddSubMenu(LPMENU pMenu,MENUCALLBACK callback,void *param);
void ShowMenu(LPMENU pMenu,int type,int x,int y);
void HideMenu(LPMENU pMenu,int hideParent);
void DestroyMenu(LPMENU pMenu);
int ProcessMenu(LPMENU pMenu,XEvent *pEvent,int *destroyed);

#endif
